@@ -1,8 +1,7 @@
 #!/usr/local/bin/perl -I ../lib
 ##############################################################################
 # 	Copyright (c) 2000 All rights reserved
-# 	Alberto Reggiori / <alberto.reggiori@jrc.it>
-# 	ISIS/RIT, Joint Research Center Ispra (I)
+#	Alberto Reggiori <areggiori@webweaving.org>
 #
 #
 # Redistribution and use in source and binary forms, with or without
@@ -45,8 +44,7 @@
 #
 # ====================================================================
 #
-# This software consists of work developed by the ISIS/RIT group on behalf
-# of the Joint Research Center of the European Commission and was originally
+# This software consists of work developed by Alberto Reggiori and was originally
 # based on public domain software written at the Stanford University Database
 # Group by Sergey Melnik.
 # For more information on the RDF API Draft work, 
@@ -55,14 +53,12 @@
 ##############################################################################
 
 use Carp;
-use Fcntl;
-use BerkeleyDB;
 use RDFStore;
 use Data::MagicTie;
 
 my $Usage =<<EOU;
 Usage is:
-    $0 [-h] -stuff <URL_or_filename> [-output_dir <valid_directoryname>] -storename <IDENTIFIER> [-style <BerkeleyDB|DB_File>] [-split <number>] [-remote] [-dbms_host <hostname>] [-dbms_port <port>] [-serialise] [-v]
+    $0 [-h] -stuff <URL_or_filename> [-output_dir <valid_directoryname>] -storename <IDENTIFIER> [-style <BerkeleyDB|DB_File|DBMS>] [-split <number>] [-dbms_host <hostname>] [-dbms_port <port>] [-serialise] [-v]
 
 Query an existing RDFStore database.
 
@@ -78,19 +74,17 @@ Query an existing RDFStore database.
 -storename <IDENTIFIER>
 		A label or identifier to identify the RDFStore database
 
-[-style <BerkeleyDB|DB_File>]
-		BerkeleyDB or DB_File store. Default is DB_File.
+[-style <BerkeleyDB|DB_File|DBMS>]
+		BerkeleyDB, DB_File or DBMS store. Default is DB_File.
 
 [-split <number>]
 		Number of DB files to split around - see Data::MagicTie(3)
-[-remote]
-		Use a remote DBMS server. Default is local. If remote it reads -dbms_host 
-		-dbms_port params
-[-dbms_host <hostname>]
-		Name or IP number of the DBMS host. Default is 'localhsot' see man dbmsd(8)
 
+[-dbms_host <hostname>]
+                Name or IP number of the DBMS host. This option makes sense only using the DBMS style; default is 'localhsot' see man dbmsd(8)
+ 
 [-dbms_port <port>]
-		TCP/IP port number of the DBMS host. Default is 1234
+                TCP/IP port number of the DBMS host. This option makes sense only using the DBMS style; default is 1234
 
 [-serialise]
                 generate strawman RDF output
@@ -102,7 +96,7 @@ print $Usage and exit if ($#ARGV<0);
 
 my $factory = new RDFStore::NodeFactory();
 
-my ($verbose,$serialise,$stuff,$storename,$style,$split,$output_dir,$remote,$dbms_host,$dbms_port);
+my ($verbose,$serialise,$stuff,$storename,$style,$split,$output_dir,$dbms_host,$dbms_port);
 my @query;
 while (defined($ARGV[0]) and $ARGV[0] =~ /^[-+]/) {
     my $opt = shift;
@@ -115,8 +109,7 @@ while (defined($ARGV[0]) and $ARGV[0] =~ /^[-+]/) {
                         ($storename =~ /\s+/) ||
                         ($storename =~ /\/$/) );
     } elsif ($opt eq '-style') {
-	$opt=shift;
-        $style = ($opt eq 'BerkeleyDB') ? $opt : undef;
+	$style=shift;
     } elsif ($opt eq '-stuff') {
         $stuff = shift;
     } elsif ($opt eq '-serialise') {
@@ -138,8 +131,6 @@ while (defined($ARGV[0]) and $ARGV[0] =~ /^[-+]/) {
                         ($output_dir eq '') ||
                         ($output_dir =~ /\s+/) ||
                         ($output_dir =~ /\/$/) );
-    } elsif ($opt eq '-remote') {
-        $remote = 1;
     } elsif ($opt eq '-dbms_host') {
         $dbms_host = shift;
     } elsif ($opt eq '-dbms_port') {
@@ -151,28 +142,15 @@ while (defined($ARGV[0]) and $ARGV[0] =~ /^[-+]/) {
 };
 
 # we assume that the RDFStore has been generated by MagicTie Style parser
-# that name DBs 'rdf/triples' and 'rdf/index'
-my $triples={};
-tie %{$triples}, 'Data::MagicTie', $output_dir.$storename.'rdf/triples',(
-					style   =>      $style,
-					dbms_host =>    $dbms_host,
-                                        dbms_port =>    $dbms_port,
-                                        Q       =>      $split,
-                                        lr      =>      $remote ) or
-	croak "Cannot tie input 'triplets' DB $!";
-
-my $triples_index = {};
-tie %{$triples_index}, 'Data::MagicTie', $output_dir.$storename.'rdf/index',(
-					style   =>      $style,
-					dbms_host =>    $dbms_host,
-                                        dbms_port =>    $dbms_port,
-                                        Q       =>      $split,
-                                        lr      =>      $remote ) or
-	croak "Cannot tie input 'triples INDEX' DB $!";
-
-my $index = new RDFStore::FindIndex($triples_index);
-my $model = new RDFStore::SetModel(undef,$triples,$index,undef);
-my $factory = new RDFStore::NodeFactory();
+my $model = new RDFStore::SetModel(
+					Name	=>	$output_dir.$storename,
+					Style   =>      $style,
+					Host =>    $dbms_host,
+                                        Port =>    $dbms_port,
+                                        Split       =>      $split,
+					Sync	=> 1 )
+	or croak "Oh dear, can not build my model :( $!";
+$factory = new RDFStore::NodeFactory();
 
 if($stuff eq '-') {
 	*STUFF=*STDIN;
@@ -181,28 +159,31 @@ if($stuff eq '-') {
 };
 
 my $stats=0;
-# input must look like (S,P,O) or (S,P,"O")
+# input must look like ("S","P","O") or ("S","P",literal("O"))
 while(<STUFF>) {
-	if(m/\(([^,]*),([^,]+),(.*)\)$/) {
-		my $s = trim($1);
-		my $p = trim($2);
-		my $o = trim($3);
-		$s =~ s/^stdin#//g;
-		$s = $factory->createResource($s);
-		$p = $factory->createResource($p);
-		if($o =~ m/\s*"([^"]+)"\s*/) {
-			$o = $factory->createLiteral($1);
-		} else {
-			$o = $factory->createResource($o);
-		};
-		my $stat = $factory->createStatement($s,$p,$o);
-		print STDERR $stat->toString,"\n"
-			if($verbose);
-		$model->add($stat);
-		$stats++;
-	};
+        if(     (m/\("([^,]*)", "([^,]+)", "(.*)"\)$/) ||
+                (m/\("([^,]*)", "([^,]+)", literal\("(.*)"\)\)$/) ) {
+                my $s = trim($1);
+                my $p = trim($2);
+                my $o = trim($3);
+                $s =~ s/^(stdin|file:[^#]+)#?(\w+:)/$2/;
+                $p =~ s/^(stdin|file:[^#]+)#?(\w+:)/$2/;
+                $s = $factory->createResource($s);
+                $p = $factory->createResource($p);
+                if($o =~ m/\s*"([^"]+)"\s*/) {
+                        $o = $factory->createLiteral($1);
+                } else {
+                        $o =~ s/^(stdin|file:[^#]+)#?(\w+:)/$2/;
+                        $o = $factory->createResource($o);
+                };
+                my $stat = $factory->createStatement($s,$p,$o);
+                print STDERR $stat->toString,"\n"
+                        if($verbose);
+                $model->add($stat);
+                $stats++;
+        };
 };
-print STDERR "Ingested '$stats' RDF statements in '$output_dir$storename' using '$sytle' style and split factor '$split'\n"
+print STDERR "Ingested '$stats' RDF statements in '$output_dir$storename' using '$style' style and split factor '$split'\n"
 	if($verbose);
 unless($stuff eq '-') {
 	close(STUFF);
@@ -212,9 +193,6 @@ if($serialise) {
         print $model->toStrawmanRDF(),"\n";
 };
 
-untie %{$triples};
-untie %{$triples_index};
-
 sub trim {
 	my $t = shift;
 	$t =~ s/^\s+//g;
